@@ -3,7 +3,7 @@
   FILE: icaltime.c
   CREATOR: eric 02 June 2000
   
-  $Id: icaltime.c,v 1.31 2004/04/02 00:08:45 jpr Exp $
+  $Id: icaltime.c,v 1.32 2004/05/25 09:34:01 rodrigo Exp $
   $Locker:  $
     
  (C) COPYRIGHT 2000, Eric Busboom, http://www.softwarestudio.org
@@ -318,8 +318,8 @@ char* set_tz(const char* tzid)
     putenv(new_tz); 
 
     /* Free any previous TZ environment string we have used. */
-    if (saved_tz)
-      free (saved_tz);
+    //if (saved_tz)
+    //  free (saved_tz);
 
     /* Save a pointer to the TZ string we just set, so we can free it later. */
     saved_tz = new_tz;
@@ -338,8 +338,8 @@ void unset_tz(char *tzstr)
     } 
 
     /* Free any previous TZ environment string we have used. */
-    if (saved_tz)
-      free (saved_tz);
+    //if (saved_tz)
+    //  free (saved_tz);
 
     /* Save a pointer to the TZ string we just set, so we can free it later.
        (This can possibly be NULL if there was no TZ to restore.) */
