@@ -132,9 +132,9 @@ e_gw_connection_dispose (GObject *object)
 	EGwConnectionPrivate *priv;
 
 	g_return_if_fail (E_IS_GW_CONNECTION (cnc));
-
+	
 	priv = cnc->priv;
-
+	printf ("gw connection dispose \n");
 	if (priv) {
 		if (priv->session_id) {
 			logout (cnc);
@@ -192,7 +192,7 @@ e_gw_connection_finalize (GObject *object)
 	g_return_if_fail (E_IS_GW_CONNECTION (cnc));
 
 	priv = cnc->priv;
-
+	printf ("gw connection finalize\n");
 	/* clean up */
 	g_free (priv);
 	cnc->priv = NULL;
@@ -498,7 +498,7 @@ e_gw_connection_get_items (EGwConnection *cnc, const char *container, const char
         SoupSoapResponse *response;
         EGwConnectionStatus status;
         SoupSoapParameter *param, *subparam;
-
+	
         g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_OBJECT);
 	
 	/* build the SOAP message */
@@ -508,9 +508,10 @@ e_gw_connection_get_items (EGwConnection *cnc, const char *container, const char
                 return E_GW_CONNECTION_STATUS_UNKNOWN;
         }
 
-        e_gw_message_write_string_parameter (msg, "container", NULL, container);
+	e_gw_message_write_string_parameter (msg, "container", NULL, container);
 	if (view)
 		e_gw_message_write_string_parameter (msg, "view", NULL, view);
+      	
 	if (filter) 
 		e_gw_filter_append_to_soap_message (filter, msg);
 	e_gw_message_write_footer (msg);
@@ -555,6 +556,75 @@ e_gw_connection_get_items (EGwConnection *cnc, const char *container, const char
         return E_GW_CONNECTION_STATUS_OK;
 }
 
+EGwConnectionStatus
+e_gw_connection_get_items_from_ids (EGwConnection *cnc, const char *container, const char *view, GPtrArray *item_ids, GList **list)
+{
+        SoupSoapMessage *msg;
+        SoupSoapResponse *response;
+        EGwConnectionStatus status;
+        SoupSoapParameter *param, *subparam;
+	int i;
+        g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_OBJECT);
+	
+	/* build the SOAP message */
+        msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "getItemsRequest");
+        if (!msg) {
+                g_warning (G_STRLOC ": Could not build SOAP message");
+                return E_GW_CONNECTION_STATUS_UNKNOWN;
+        }
+
+	e_gw_message_write_string_parameter (msg, "container", NULL, container);
+	if (view)
+		e_gw_message_write_string_parameter (msg, "view", NULL, view);
+      	soup_soap_message_start_element (msg, "items", NULL, NULL);
+	for (i = 0; i < item_ids->len; i ++) {
+		char *id = g_ptr_array_index (item_ids, i);
+		e_gw_message_write_string_parameter (msg, "item", NULL, id);
+	}
+	soup_soap_message_end_element (msg);
+
+	e_gw_message_write_footer (msg);
+
+        /* send message to server */
+        response = e_gw_connection_send_message (cnc, msg);
+        if (!response) {
+                g_object_unref (msg);
+                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+        }
+
+        status = e_gw_connection_parse_response_status (response);
+        if (status != E_GW_CONNECTION_STATUS_OK) {
+		g_object_unref (response);
+                g_object_unref (msg);
+		return status;
+	}
+
+	/* if status is OK - parse result. return the list */	
+	param = soup_soap_response_get_first_parameter_by_name (response, "items");
+        if (!param) {
+                g_object_unref (response);
+                g_object_unref (msg);
+                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+        }
+	
+        /* parse these parameters into ecalcomponents*/
+        for (subparam = soup_soap_parameter_get_first_child_by_name (param, "item");
+	     subparam != NULL;
+	     subparam = soup_soap_parameter_get_next_child_by_name (subparam, "item")) {
+		EGwItem *item;
+
+		item = e_gw_item_new_from_soap_parameter (container, subparam);
+		if (item)
+			*list = g_list_append (*list, item);
+        }
+               
+	/* free memory */
+        g_object_unref (response);
+	g_object_unref (msg);
+
+        return E_GW_CONNECTION_STATUS_OK;
+}
+
 EGwConnectionStatus
 e_gw_connection_get_deltas ( EGwConnection *cnc, GSList **adds, GSList **deletes, GSList **updates)
 {
@@ -569,7 +639,7 @@ e_gw_connection_get_deltas ( EGwConnection *cnc, GSList **adds, GSList **deletes
          msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "getDeltaRequest"); 
          if (!msg) { 
                  g_warning (G_STRLOC ": Could not build SOAP message"); 
-		 g_object_unref (cnc); 
+		 // g_object_unref (cnc); 
                  return E_GW_CONNECTION_STATUS_UNKNOWN; 
          } 
         
@@ -582,7 +652,7 @@ e_gw_connection_get_deltas ( EGwConnection *cnc, GSList **adds, GSList **deletes
          response = e_gw_connection_send_message (cnc, msg); 
          if (!response) { 
                  g_object_unref (msg); 
-		 g_object_unref (cnc); 
+		 // g_object_unref (cnc); 
                  return E_GW_CONNECTION_STATUS_INVALID_RESPONSE; 
          } 
 
@@ -590,7 +660,7 @@ e_gw_connection_get_deltas ( EGwConnection *cnc, GSList **adds, GSList **deletes
          if (status != E_GW_CONNECTION_STATUS_OK) { 
  		g_object_unref (response); 
 		g_object_unref (msg); 
- 		g_object_unref (cnc); 
+		//	g_object_unref (cnc); 
  		return status; 
  	} 
 
@@ -599,13 +669,13 @@ e_gw_connection_get_deltas ( EGwConnection *cnc, GSList **adds, GSList **deletes
          if (!param) { 
                  g_object_unref (response); 
                  g_object_unref (msg); 
-		 g_object_unref (cnc); 
+		 // g_object_unref (cnc); 
                  return E_GW_CONNECTION_STATUS_INVALID_RESPONSE; 
          } 
 	
          if (!g_ascii_strcasecmp ( soup_soap_parameter_get_string_value (param), "0")) { 
                  g_message ("No deltas"); 
-		 g_object_unref (cnc); 
+		 // g_object_unref (cnc); 
                  return E_GW_CONNECTION_STATUS_OK; 
          }                 
 
@@ -613,7 +683,7 @@ e_gw_connection_get_deltas ( EGwConnection *cnc, GSList **adds, GSList **deletes
          if (!param) { 
                  g_object_unref (response); 
                  g_object_unref (msg); 
-		 g_object_unref (cnc); 
+		 // g_object_unref (cnc); 
                  return E_GW_CONNECTION_STATUS_INVALID_RESPONSE; 
          } 
         
@@ -631,7 +701,7 @@ e_gw_connection_get_deltas ( EGwConnection *cnc, GSList **adds, GSList **deletes
                                  if (!param_id) { 
                                          g_object_unref (response); 
                                          g_object_unref (msg); 
-                                         g_object_unref (cnc); 
+					 // g_object_unref (cnc); 
                                  } 
                                  uid = (char *)soup_soap_parameter_get_string_value (param_id); 
                                  /*if (!e_cal_backend_cache_remove_component (cache, uid, NULL)) 
@@ -653,7 +723,7 @@ e_gw_connection_get_deltas ( EGwConnection *cnc, GSList **adds, GSList **deletes
                                 if (!item) { 
                                          g_object_unref (response); 
                                          g_object_unref (msg); 
-                                         g_object_unref (cnc); 
+					 // g_object_unref (cnc); 
  					return E_GW_CONNECTION_STATUS_INVALID_RESPONSE; 
                                  } 
                                  /*if (!e_cal_backend_cache_put_component (cache, comp)) 
@@ -913,6 +983,41 @@ e_gw_connection_remove_item (EGwConnection *cnc, const char *container, const ch
 	return status;
 }
 
+EGwConnectionStatus
+e_gw_connection_remove_items (EGwConnection *cnc, const char *container, GList *item_ids)
+{
+	SoupSoapMessage *msg;
+	SoupSoapResponse *response;
+	EGwConnectionStatus status;
+
+	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_CONNECTION);
+
+	/* build the SOAP message */
+	msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "removeItemsRequest");
+	if (container && *container)
+		e_gw_message_write_string_parameter (msg, "container", NULL, container);
+
+	soup_soap_message_start_element (msg, "items", NULL, NULL);
+	for (; item_ids != NULL; item_ids = g_list_next (item_ids))
+		e_gw_message_write_string_parameter (msg, "item", NULL, item_ids->data);
+	soup_soap_message_end_element (msg);
+	e_gw_message_write_footer (msg);
+
+	/* send message to server */
+	response = e_gw_connection_send_message (cnc, msg);
+	if (!response) {
+		g_object_unref (msg);
+		return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+	}
+
+	status = e_gw_connection_parse_response_status (response);
+
+	/* free memory */
+	g_object_unref (response);
+	g_object_unref (msg);
+
+	return status;
+}
 const char *
 e_gw_connection_get_uri (EGwConnection *cnc)
 {
@@ -994,6 +1099,24 @@ timet_from_string (const char *str)
 	return mktime (&date);
 }
 
+char *
+e_gw_connection_format_date_string (const char *dtstring)
+{
+        char *str2;
+        int i, j, len = strlen (dtstring);
+	
+        str2 = g_malloc0 (len);
+        for (i = 0,j = 0; i < len; i++) {
+                if ((dtstring[i] != '-') && (dtstring[i] != ':')) {
+			str2[j] = dtstring[i];
+			j++;
+                }
+        }
+
+	str2[j] = '\0';
+        return str2;
+}
+
 time_t
 e_gw_connection_get_date_from_string (const char *dtstring)
 {
@@ -1049,7 +1172,7 @@ e_gw_connection_create_book (EGwConnection *cnc, char *book_name, char**id)
 	if (param)
 		value = soup_soap_parameter_get_string_value (param);
 	if (value)
-		*id = g_strdup (value);
+		*id = value;
 
 	status = E_GW_CONNECTION_STATUS_OK;	
 	return status;	
@@ -1062,7 +1185,7 @@ e_gw_connection_get_address_book_list (EGwConnection *cnc, GList **container_lis
 	SoupSoapResponse *response;
         EGwConnectionStatus status;
 	SoupSoapParameter *param;
-	SoupSoapParameter *is_personal_param;
+	SoupSoapParameter *type_param;
 	char *value;
 	
 	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_UNKNOWN);
@@ -1106,14 +1229,23 @@ e_gw_connection_get_address_book_list (EGwConnection *cnc, GList **container_lis
 			container = e_gw_container_new_from_soap_parameter (subparam);
 			if (container) {
 				*container_list = g_list_append (*container_list, container);
-				is_personal_param = soup_soap_parameter_get_first_child_by_name (subparam, "isPersonal");
+				type_param = soup_soap_parameter_get_first_child_by_name (subparam, "isPersonal");
 				value = NULL;
-				if (is_personal_param)
-					value = soup_soap_parameter_get_string_value (is_personal_param);
+				if (type_param)
+					value = soup_soap_parameter_get_string_value (type_param);
 				if (value && g_str_equal(value , "1"))
 					e_gw_container_set_is_writable (container, TRUE);
 				else 
 					e_gw_container_set_is_writable (container, FALSE);
+				g_free (value);
+				value = NULL;
+			        type_param = soup_soap_parameter_get_first_child_by_name (subparam, "isFrequentContacts");
+				if (type_param)
+                                        value = soup_soap_parameter_get_string_value (type_param);
+                                if (value && g_str_equal(value , "1"))
+                                        e_gw_container_set_is_frequent_contacts (container, TRUE);
+                                                                        
+				g_free (value);
 					
 			}
 				     
@@ -1210,11 +1342,15 @@ e_gw_connection_get_categories (EGwConnection *cnc, GHashTable *categories_by_id
 		if (second_level_child)
 			name = soup_soap_parameter_get_string_value (second_level_child);
 		if (id && name) {
-			id = g_strsplit (id, "@", -1) [0];
+			char **components = g_strsplit (id, "@", -1);
+			g_free (id);
+			id = components[0];
 			if (categories_by_id) 
 				g_hash_table_insert (categories_by_id, g_strdup (id), g_strdup (name));
 			if (categories_by_name) 
 				g_hash_table_insert (categories_by_name, g_strdup (name), g_strdup (id));
+			g_strfreev (components);
+			g_free (name);
 		}
 		
         }
@@ -1228,3 +1364,86 @@ e_gw_connection_get_categories (EGwConnection *cnc, GHashTable *categories_by_id
 
 }
 
+EGwConnectionStatus 
+e_gw_connection_add_members (EGwConnection *cnc, const char *group_id, GList *member_ids)
+{
+	SoupSoapMessage *msg;
+	SoupSoapResponse *response;
+        EGwConnectionStatus status;
+	
+	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_UNKNOWN);
+	g_return_val_if_fail (member_ids != NULL, E_GW_CONNECTION_STATUS_UNKNOWN);
+	g_return_val_if_fail (group_id != NULL, E_GW_CONNECTION_STATUS_UNKNOWN);
+	
+	 msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "addMembersRequest");
+        if (!msg) {
+                g_warning (G_STRLOC ": Could not build SOAP message");
+                return E_GW_CONNECTION_STATUS_UNKNOWN;
+        }
+	e_gw_message_write_string_parameter (msg, "container", NULL, group_id);
+	soup_soap_message_start_element (msg, "members", NULL, NULL);
+
+	for (; member_ids != NULL; member_ids = member_ids = g_list_next (member_ids)) {
+		soup_soap_message_start_element (msg, "member", NULL, NULL);
+		e_gw_message_write_string_parameter (msg, "id", NULL, member_ids->data);
+		soup_soap_message_end_element(msg);
+	}
+	
+	soup_soap_message_end_element(msg);
+	e_gw_message_write_footer (msg);
+	response = e_gw_connection_send_message (cnc, msg);
+        if (!response) {
+                g_object_unref (msg);
+                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+        }
+
+        status = e_gw_connection_parse_response_status (response);
+	g_object_unref (response);
+	g_object_unref (msg);
+	return status;
+	
+
+}
+
+EGwConnectionStatus 
+e_gw_connection_remove_members (EGwConnection *cnc, const char *group_id, GList *member_ids)
+{
+	
+	SoupSoapMessage *msg;
+	SoupSoapResponse *response;
+        EGwConnectionStatus status;
+	
+	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_UNKNOWN);
+	g_return_val_if_fail (member_ids != NULL, E_GW_CONNECTION_STATUS_UNKNOWN);
+	g_return_val_if_fail (group_id != NULL, E_GW_CONNECTION_STATUS_UNKNOWN);
+	
+	msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "removeMembersRequest");
+        if (!msg) {
+                g_warning (G_STRLOC ": Could not build SOAP message");
+                return E_GW_CONNECTION_STATUS_UNKNOWN;
+        }
+	e_gw_message_write_string_parameter (msg, "container", NULL, group_id);
+	soup_soap_message_start_element (msg, "members", NULL, NULL);
+
+	for (; member_ids != NULL; member_ids = member_ids = g_list_next (member_ids)) {
+		soup_soap_message_start_element (msg, "member", NULL, NULL);
+		e_gw_message_write_string_parameter (msg, "id", NULL, member_ids->data);
+		soup_soap_message_end_element(msg);
+	}
+
+	soup_soap_message_end_element(msg);
+	e_gw_message_write_footer (msg);
+	response = e_gw_connection_send_message (cnc, msg);
+        if (!response) {
+                g_object_unref (msg);
+                return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+        }
+
+        status = e_gw_connection_parse_response_status (response);
+	g_object_unref (response);
+	g_object_unref (msg);
+	return status;
+
+
+
+}
